\name{smartpred}
\alias{smartpred}
\title{
Smart Prediction 
}
\description{
  Data-dependent parameters in formula terms
  can cause problems in when predicting.
The \code{smartpred} library saves data-dependent parameters on the
object so that the bug is fixed. The \code{lm} and \code{glm} functions
have been fixed properly. The \code{VGAM} library automatically comes with
smart prediction.
}
\usage{
lm()
glm()
ns()
bs()
poly()
scale()
}
\value{
  Returns the usual object, but with one list/slot component called
  \code{smart.prediction} containing any data-dependent parameters.
}
\section{Side Effects}{

The variables
\code{.max.smart},
\code{.smart.prediction} and 
\code{.smart.prediction.counter}
are created in the workspace while the model is being fitted.
These are deleted after the model has been fitted, however, if
there is an error in the model fitting function or the 
fitting model is killed (e.g., by typing control-C)
then these variables will be left in the workspace. 
At the beginning of model fitting, these variables are
deleted if present in the workspace. 

During prediction, the variables
\code{.smart.prediction} and 
\code{.smart.prediction.counter}
are reconstructed and read by the smart functions when the model
frame is re-evaluated. 
After prediction, these variables are deleted. 

If the modelling function is used with \code{smart=FALSE} then
smart prediciton will not be used, and the results should
match with the original R functions. 

}
\details{
  R version 1.6.0 introduced a fix for the prediction
  problem. However, it does not work all the time,
  e.g., for terms such as
  \code{I(poly(x, 3))}, 
  \code{bs(scale(x), 3)}, 
  \code{scale(scale(x))}.
  See the example below.
Smart prediction, however, will always work, albeit, not so elegantly.

The basic idea is that the functions in the formula are now smart, and 
the modelling functions make use of these smart functions. 
Smart prediction works in two ways: using \code{smart.expression},
or using a combination of \code{put.smart()} and \code{get.smart()}.
}

\author{T. W. Yee and T. J. Hastie}
%\note{
%}

\seealso{
  \code{\link{get.smart.prediction}},
  \code{\link{get.smart}},
  \code{\link{put.smart}},
  \code{\link{smart.expression}},
  \code{\link{smart.mode.is}},
  \code{\link{setup.smart}},
  \code{\link{wrapup.smart}}.
  Data-dependent functions to watch out for include
  \code{\link{scale}}, 
  \code{\link{poly}}, 
  \code{ns}, 
  \code{bs}.
The functions \code{ns} and \code{bs} are in the
\code{splines} library, and this library is automatically
loaded in because it contains compiled code that 
\code{ns} and \code{bs} call.

The website \url{http://www.stat.auckland.ac.nz/~yee}
contains more information such as how to write a
smart function, and other technical details.

The functions \code{vglm},
\code{rrvglm} and \code{vgam} in the \code{VGAM}
library are modelling functions that use smart prediction. 

}
\examples{
# Create some data first
n = 20
set.seed(86) # For reproducibility of the random numbers
x = sort(runif(n)) 
y = sort(runif(n))
\dontrun{library(splines)   # To get ns()
}

# This will work for R 1.6.0 and later 
fit = lm(y ~ ns(x, df=5))
\dontrun{plot(x, y)
lines(x, fitted(fit))
newx = seq(0, 1, len=n)
points(newx, predict(fit, data.frame(x=newx)), type="b", col=2, err=-1)
}

# The following fails for R 1.6.x and later but works with smart prediction
\dontrun{fit = lm(y ~ ns(scale(x), df=5))
fit$smart.prediction
plot(x, y)
lines(x, fitted(fit))
newx = seq(0, 1, len=n)
points(newx, predict(fit, data.frame(x=newx)), type="b", col=2, err=-1)
}
}
%\keyword{smart}
\keyword{models}
\keyword{regression}
\keyword{programming}



